/*
 * $Id: pkgwidgetimpl.cc,v 1.2 2000/12/29 20:50:48 twschulz Exp $
 * Copyright (c) 2000 Trenton Schulz
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer,
 *    without modification, immediately at the beginning of the file.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
 */


#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <cstdlib>
#include <qdir.h>
#include <qpushbutton.h>
#include <qlistbox.h>
#include <qmessagebox.h>
#include <qstatusbar.h>
#include <qapplication.h>

#include "pkgwidgetimpl.h"
#include "pkgviewimpl.h"

PkgWidgetImpl::PkgWidgetImpl(const char* strNewDB, QWidget* parent, 
                             const char* name, WFlags fl)
	: PkgWidget(parent, name, fl),
      strDB(strNewDB)
{
//	barPkg = new QStatusBar(this, "Status Bar");
//	CHECK_PTR(barPkg);
	//barPkg->message("Initializing", 5000);
	QObject::connect( cmdQuit,
	                  SIGNAL(clicked()),
	                  qApp,
                      SLOT(quit()) );
}

PkgWidgetImpl::~PkgWidgetImpl()
{
}

void PkgWidgetImpl::init(void)
{
	int i, maxPorts;
	QDir dirPkg;
	dirPkg.setPath(strDB);
	dirPkg.setFilter(QDir::Dirs);
	dirPkg.setSorting(QDir::Name);

	// insert the list into the box (in theory this could take a while)
	//barPkg->message("Reading Packages");

	// clear the list to make sure there's not any strange cruft in it
	listPkg->clear();
	maxPorts = dirPkg.count();
	for ( i = 2; i < maxPorts; i++)
	{
		listPkg->insertItem(dirPkg[i]);
	}
	listPkg->setMinimumSize(listPkg->sizeHint());
	//barPkg->message("Ready");
}
	
void PkgWidgetImpl::slotPkgDelete(void)
{
	bool bForce;
	pid_t pid;
	int status;
	QString strRequired, 
            strPkg;
	QFile fRequired;
	bForce = false;

	// Let's try to be a little smart on this before we fork...
	strPkg = listPkg->currentText();
	strRequired = strDB + "/" + strPkg + "/+REQUIRED_BY";

	if (QFile::exists(strRequired))
	{
		// open the file and see if it's empty or actually contains data
		fRequired.setName(strRequired);
		if (fRequired.size() > 0)
		{
			switch ( QMessageBox::warning(this, "Dependencies",
				"This package is required by other packages.\n"
				"Do you really want to force this delete?",
				"&Yes", "&No", "&Show Dependencies", 0, 1) )
			{
				case 0:
					bForce = true;
					break;
				case 1:
					return;
				case 2:
					slotPkgRequired();
					return;
			}
		}
	}

	pid = fork();
	
	if (pid < 0)
	{
		qDebug("fork error\n");
	}
	else if (pid == 0)
	{
		if (bForce)
		{
			execlp("/usr/sbin/pkg_delete", "pkg_delete", "-f", 
			       strPkg.latin1(), NULL);
		}
		else
		{
			execlp("/usr/sbin/pkg_delete", "pkg_delete", 
		   	       strPkg.latin1(), NULL);
		}
		// something bad happened if you got here...
		exit(-1);
	}
	else
	{
		//barPkg->message("Removing Package");
		wait(&status);
		// do stuff based on the return value
		switch(status)
		{
			case 0:	
			// remove the item from the list
				listPkg->removeItem(listPkg->currentItem());
				break;
			default:
				// do nothing
				break;
		}
		//barPkg->message("Ready");
	}
}

void PkgWidgetImpl::slotPkgDesc(void)
{
	QString strDesc;
	strDesc = strDB + "/" + listPkg->currentText() + "/+DESC";
	PkgViewImpl frmDesc(strDesc);
	frmDesc.exec();
}

void PkgWidgetImpl::slotPkgContents(void)
{
	// change the status bar to read new contents
	QString strContents;
	//barPkg->message("Reading Contents");
	strContents = strDB + "/" + listPkg->currentText() + "/+CONTENTS";
	PkgViewImpl frmContents(strContents);
	frmContents.exec();
	//barPkg->message("Ready");
}

void PkgWidgetImpl::slotPkgRequired(void)
{
	QString strRequired;
	strRequired = strDB + "/" + listPkg->currentText() + "/+REQUIRED_BY";
	PkgViewImpl frmRequired(strRequired);
	frmRequired.exec();
}

void PkgWidgetImpl::slotEnable(void)
{
	cmdRemove->setEnabled(true);
	cmdDesc->setEnabled(true);
	cmdContents->setEnabled(true);
}

void PkgWidgetImpl::hideRemove(void)
{
        cmdRemove->hide();
}

void PkgWidgetImpl::showRemove(void)
{
        cmdRemove->show();
}
